#!/bin/sh
# Bryant Hansen

ME="$(basename "$0")"
USAGE="$ME"

SYNC_SRC_DIR="/tmp/logs"
CONF_FILE="/etc/logfile_uploader.conf"

# 50 MB
MAX_SIZE_MB=50

DebugMessage() {
    local LOG_RECORD_HEADER="`date "+%Y%m%d_%H%M%S"` (${ME}):"
    echo "${LOG_RECORD_HEADER} $*" >&2
}

# Determine if a log file is rotated via the logrotate utility
#   check if the filename ends with a . followed by 1 or more numbers
is_logrotated() {
    expr match "$1" ".*\.[0-9][0-9]*$" > /dev/null && return 0 || return 1
}

link_single_file() {
    if [[ -f "$1" ]] ; then
        dir="$(dirname "$1")"
        destdir="${SYNC_SRC_DIR}/$(dirname "$1")"
        if [[ ! -d "$destdir" ]] ; then
                echo "mkdir -p \"${destdir}\""
        fi
        size="$(stat -c "%s" "$1")"
        sizeMB="$(expr $size / 1000000)"
        if [[ $sizeMB -gt $MAX_SIZE_MB ]] ; then
            DebugMessage "link_single_file ERROR: $1 is $size bytes.  Not uploading."
        else
            echo "ln -s \"${1}\" \"${SYNC_SRC_DIR}/${1}\""
        fi
    else
        DebugMessage "$1 not found.  The system does not appear to be properly configured."
    fi
}

link_historized_file() {
    if [[ -f "$1" ]] ; then
        dir="$(dirname "$1")"
        destdir="${SYNC_SRC_DIR}/$(dirname "$1")"
        if [[ ! -d "$destdir" ]] ; then
                echo "mkdir -p \"${destdir}\""
        fi
        string="$(cat "$1")"
        if which stat >/dev/null 2>&1 ; then
            dt=$(stat -c "%Y" "$1")
            if [[ "$dt" ]] ; then
                string="$(echo $string $dt)"
            fi
        fi
        md5="$(echo \"${string}\" | md5sum | cut -f 1 -d ' ')"
        size="$(stat -c "%s" "$1")"
        sizeMB="$(expr $size / 1000000)"
        if [[ $sizeMB -gt $MAX_SIZE_MB ]] ; then
            DebugMessage "link_historized_file ERROR: $1 is $size bytes.  Not uploading."
        else
            echo "ln -s \"${1}\" \"${SYNC_SRC_DIR}/${1}.${md5}\""
        fi
    else
        DebugMessage "$1 not found.  The system does not appear to be properly configured."
    fi
}


if [[ ! -f "$CONF_FILE" ]] ; then
	DebugMessage "ERROR: configuration file $CONF_FILE not found."
	DebugMessage "Exiting abnormally."
	exit 2
fi
. "$CONF_FILE"

if [[ ! "$SYNC_SRC_DIR" ]] ; then
	DebugMessage "ERROR: SYNC_SRC_DIR not specified"
	DebugMessage "Exiting abnormally."
	exit 3
fi

if [[ -d "$SYNC_SRC_DIR" ]] ; then
	DebugMessage "SYNC_SRC_DIR $SYNC_SRC_DIR already exists.  Exiting without doing anything."
	exit 0
fi
echo "mkdir -p \"${SYNC_SRC_DIR}\""

# start by verifying that there's a link to apartments.xml
ent=/usr/share/dss/data/apartment.xml
if [[ -f "$ent" ]] ; then
    dir="$(dirname "$ent")"
    destdir="${SYNC_SRC_DIR}/$(dirname "$ent")"
    if [[ ! -d "$destdir" ]] ; then
            echo "mkdir -p \"${destdir}\""
    fi
    md5="$(cat "$ent" | md5sum | cut -f 1 -d ' ')"
    size="$(stat -c "%s" "$ent")"
    sizeMB="$(expr $size / 1000000)"
    if [[ $sizeMB -gt $MAX_SIZE_MB ]] ; then
        DebugMessage "link_historized_file ERROR: $ent is $size bytes.  Not uploading."
    else
        echo "ln -s \"${ent}\" \"${SYNC_SRC_DIR}/${ent}.${md5}\""
    fi
else
    DebugMessage "$ent not found.  The system does not appear to be properly configured."
fi

link_single_file /etc/dSS11-image-version
link_single_file /usr/share/dss/data/security.xml
link_single_file /usr/share/dss/data/config.d/location.xml
link_historized_file /etc/timezone

# TODO: split this off into a new file
opkg_installed_dir="/var/log/opkg-installed"
if [[ ! -d "${opkg_installed_dir}" ]] ; then
    echo "mkdir \"${opkg_installed_dir}\""
fi
opkg_installed_tmpfile="/tmp/opkg_installed.lst.gz"
opkg list-installed 2>&1 | gzip > "${opkg_installed_tmpfile}"
if [[ ! -f "${opkg_installed_tmpfile}" ]] ; then
    DebugMessage "WARNING: failed to create opkg-installed tmpfile"
else
    size="$(stat -c "%s" "$opkg_installed_tmpfile")"
    sizeMB="$(expr $size / 1000000)"
    if [[ $sizeMB -gt $MAX_SIZE_MB ]] ; then
        DebugMessage "ERROR: opkg_installed_tmpfile $opkg_installed_tmpfile is $size bytes.  Not uploading."
    else
        md5=$(zcat "${opkg_installed_tmpfile}" | md5sum | cut -f 1 -d ' ')
        opkg_installed_file="${opkg_installed_dir}/opkg_installed.${md5}.gz"
        if [[ ! -f "${opkg_installed_file}" ]] ; then
            echo "mv \"${opkg_installed_tmpfile}\" \"${opkg_installed_file}\""
        fi
    fi
fi
rm "${opkg_installed_tmpfile}" 2>/dev/null

# TODO: split this off into a new file
traceroute_logfilename="traceroute.log.gz"
traceroute_logdir="/var/log/traceroute"
IP="$(get_ip_address.sh)"
if [[ "$IP" ]] ; then
    traceroute_logfilename="traceroute_${IP}.log.gz"
else
    DebugMessage "$ME WARNING: failed to get IP address of host"
fi
if [[ ! -d "${traceroute_logdir}" ]] ; then
    echo "mkdir \"${traceroute_logdir}\""
fi
traceroute_logfile="${traceroute_logdir}/${traceroute_logfilename}"
if [[ ! -f "$traceroute_logfile" ]] ; then
    rm "${traceroute_logdir}/traceroute*.log.gz"
    echo "traceroute -m 5 -q 2 ${SERVER} 2>&1 | gzip > \"${traceroute_logfile}\""
fi
size="$(stat -c "%s" "$traceroute_logfile")"
sizeMB="$(expr $size / 1000000)"
if [[ $sizeMB -gt $MAX_SIZE_MB ]] ; then
    DebugMessage "ERROR: traceroute_logfile $traceroute_logfile is $size bytes.  Not uploading."
    rm "$traceroute_logfile"
fi

find \
    /var/log \
    /usr/share/dss/data/savedprops | \
    while read ent ; do
        if [[ -d "$ent" ]] ; then
            if [[ ! -d "$SYNC_SRC_DIR"/"$ent" ]] ; then
                d="$SYNC_SRC_DIR"/"$ent"
                [[ ! -d "$d" ]] && echo "mkdir -p \"${d}\""
            fi
        fi
        if [[ -f "$ent" ]] ; then
            size="$(stat -c "%s" "$ent")"
            sizeMB="$(expr $size / 1000000)"
            if [[ $sizeMB -gt $MAX_SIZE_MB ]] ; then
                DebugMessage "ERROR: $ent is $size bytes.  Not uploading."
                continue
            fi
            dir="$(dirname "$ent")"
            d="$SYNC_SRC_DIR"/"$dir"
            [[ ! -d "$d" ]] && echo "mkdir -p \"${d}\""
            # detect if this is a logrotated file
            if is_logrotated "$ent" ; then
                #echo "$ent is a logrotate file" >&2
                if which stat >/dev/null 2>&1 ; then
                    dt=$(stat -c "%X" "$ent")
                    if [[ ! "$dt" ]] ; then
                        # echo "failed to get the file date for some unknown reason." >&2
                        # echo "using the first 8 digits of the md5 instead" >&2
                        md5="$(md5sum "$ent" | cut -f 1 -d ' ')"
                        md5=${md5:0:8}
                        dt=$md5
                    fi
                    dest="${SYNC_SRC_DIR}/${ent%\.[0-9]*}.${dt}"
                else
                    md5="$(md5sum "$ent" | cut -f 1 -d ' ')"
                    dest="${SYNC_SRC_DIR}/${ent%\.[0-9]*}.${md5}"
                fi
                echo "ln -s \"${ent}\" \"${dest}\""
            else
                #echo "$ent is not a logrotate file" >&2
                dest="${SYNC_SRC_DIR}/${ent}"
                echo "ln -s \"${ent}\" \"${dest}\""
            fi
        fi
    done

echo "mkdir \"${SYNC_SRC_DIR}\"/smemcap"
echo "smemcap | gzip > \"${SYNC_SRC_DIR}\"/smemcap/$(date "+%Y%m%d_%H%M%S").tar.gz"

echo "mkdir \"${SYNC_SRC_DIR}\"/df"
echo "df 2>&1 | gzip > \"${SYNC_SRC_DIR}\"/df/$(date "+%Y%m%d_%H%M%S").log.gz"

echo "mkdir \"${SYNC_SRC_DIR}\"/ps"
if ps auwx > /dev/null 2>&1 ; then
    # full (procps) version of ps
    echo "ps auwx 2>&1 | gzip > \"${SYNC_SRC_DIR}\"/ps/$(date "+%Y%m%d_%H%M%S").log.gz"
else
    # busybox version of ps
    echo "ps w 2>&1 | gzip > \"${SYNC_SRC_DIR}\"/ps/$(date "+%Y%m%d_%H%M%S").log.gz"
fi

TOP_ITERATIONS=3
echo "mkdir \"${SYNC_SRC_DIR}\"/top"
# for the busybox version of top
echo "top -b -n${TOP_ITERATIONS} 2>&1 | gzip > \"${SYNC_SRC_DIR}\"/top/$(date "+%Y%m%d_%H%M%S").log.gz"

# metering data
if [[ -d "/usr/share/dss/data/metering" ]] ; then
    for f in /usr/share/dss/data/metering/*.rrd ; do
        DebugMessage "Processing metering data from ${f}..."
        if [[ ! -f "$f" ]] ; then
            DebugMessage "ERROR: $f not found.  continuing without processing metering data"
            continue
        fi
        dts=$(date "+%s")
        dts2=$(expr $dts / 60 \* 60)
        outdir="${SYNC_SRC_DIR}/metering/$(basename "${f%.rrd}")"
        [[ ! -d "$outdir" ]] && echo "mkdir -p \"${outdir}\""
        echo "rrdtool flushcached --daemon unix:/var/run/rrdcached.sock \"${f}\""
        echo "rrdtool \
            xport \
            --daemon unix:/var/run/rrdcached.sock DEF:vname=${f}:power:AVERAGE XPORT:vname \
            --step 60 \
            --end $dts2 \
            --start end-12h \
            --maxrows 1200 | \
            gzip -9c > \"${outdir}\"/$(date "+%Y%m%d_%H%M%S").xml.gz"
    done
else
    DebugMessage "WARNING: metering data directory /usr/share/dss/data/metering does not exist.  No metering data to upload."
fi

# tighten up the security
echo "chmod -R go-rwx \"${SYNC_SRC_DIR}\""
